@@ -1,26 +1,22 @@
+use ruff_formatter::write;
+use ruff_python_ast::AnyNodeRef;
+use ruff_python_ast::ExprName;
+
 use crate::expression::parentheses::{NeedsParentheses, OptionalParentheses};
 use crate::prelude::*;
-use crate::FormatNodeRule;
-use ruff_formatter::{write, FormatContext};
-use ruff_python_ast::node::AnyNodeRef;
-use ruff_python_ast::ExprName;
 
 #[derive(Default)]
 pub struct FormatExprName;
 
 impl FormatNodeRule<ExprName> for FormatExprName {
     fn fmt_fields(&self, item: &ExprName, f: &mut PyFormatter) -> FormatResult<()> {
-        let ExprName { id, range, ctx: _ } = item;
-
-        debug_assert_eq!(
-            id.as_str(),
-            f.context()
-                .source_code()
-                .slice(*range)
-                .text(f.context().source_code())
-        );
-
-        write!(f, [source_text_slice(*range, ContainsNewlines::No)])
+        let ExprName {
+            id: _,
+            range,
+            ctx: _,
+            node_index: _,
+        } = item;
+        write!(f, [source_text_slice(*range)])
     }
 }
 
@@ -30,22 +26,21 @@ impl NeedsParentheses for ExprName {
         _parent: AnyNodeRef,
         _context: &PyFormatContext,
     ) -> OptionalParentheses {
-        OptionalParentheses::Never
+        OptionalParentheses::BestFit
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use ruff_python_ast::Ranged;
-    use ruff_python_parser::parse_program;
-    use ruff_text_size::{TextRange, TextSize};
+    use ruff_python_parser::parse_module;
+    use ruff_text_size::{Ranged, TextRange, TextSize};
 
     #[test]
     fn name_range_with_comments() {
-        let source = parse_program("a # comment", "file.py").unwrap();
+        let module = parse_module("a # comment").unwrap();
 
-        let expression_statement = source
-            .body
+        let expression_statement = module
+            .suite()
             .first()
             .expect("Expected non-empty body")
             .as_expr_stmt()
