@@ -1,12 +1,9 @@
-use ruff_formatter::{format_args, write, Buffer, FormatResult};
-use ruff_python_ast::node::AnyNodeRef;
+use ruff_formatter::{Buffer, FormatResult, format_args, write};
+use ruff_python_ast::AnyNodeRef;
 use ruff_python_ast::ExprSetComp;
 
-use crate::context::PyFormatContext;
-use crate::expression::parentheses::{parenthesized, NeedsParentheses, OptionalParentheses};
+use crate::expression::parentheses::{NeedsParentheses, OptionalParentheses, parenthesized};
 use crate::prelude::*;
-use crate::AsFormat;
-use crate::{FormatNodeRule, PyFormatter};
 
 #[derive(Default)]
 pub struct FormatExprSetComp;
@@ -15,6 +12,7 @@ impl FormatNodeRule<ExprSetComp> for FormatExprSetComp {
     fn fmt_fields(&self, item: &ExprSetComp, f: &mut PyFormatter) -> FormatResult<()> {
         let ExprSetComp {
             range: _,
+            node_index: _,
             elt,
             generators,
         } = item;
@@ -26,7 +24,7 @@ impl FormatNodeRule<ExprSetComp> for FormatExprSetComp {
         });
 
         let comments = f.context().comments().clone();
-        let dangling = comments.dangling_comments(item);
+        let dangling = comments.dangling(item);
 
         write!(
             f,
@@ -35,18 +33,13 @@ impl FormatNodeRule<ExprSetComp> for FormatExprSetComp {
                 &group(&format_args!(
                     group(&elt.format()),
                     soft_line_break_or_space(),
-                    &joined
+                    joined
                 )),
                 "}"
             )
             .with_dangling_comments(dangling)]
         )
     }
-
-    fn fmt_dangling_comments(&self, _node: &ExprSetComp, _f: &mut PyFormatter) -> FormatResult<()> {
-        // Handled as part of `fmt_fields`
-        Ok(())
-    }
 }
 
 impl NeedsParentheses for ExprSetComp {
