@@ -318,6 +318,9 @@ pub enum KnownModule {
     TypingExtensions,
     Typing,
     Sys,
+    Os,
+    Tempfile,
+    Pathlib,
     Abc,
     Dataclasses,
     Collections,
@@ -347,6 +350,9 @@ impl KnownModule {
             Self::Typeshed => "_typeshed",
             Self::TypingExtensions => "typing_extensions",
             Self::Sys => "sys",
+            Self::Os => "os",
+            Self::Tempfile => "tempfile",
+            Self::Pathlib => "pathlib",
             Self::Abc => "abc",
             Self::Dataclasses => "dataclasses",
             Self::Collections => "collections",
