@@ -7,7 +7,7 @@ use config::SystemKind;
 use parser as test_parser;
 use ruff_db::Db as _;
 use ruff_db::diagnostic::{Diagnostic, DisplayDiagnosticConfig};
-use ruff_db::files::{File, system_path_to_file};
+use ruff_db::files::{File, FileRootKind, system_path_to_file};
 use ruff_db::panic::catch_unwind;
 use ruff_db::parsed::parsed_module;
 use ruff_db::system::{DbWithWritableSystem as _, SystemPath, SystemPathBuf};
@@ -18,8 +18,9 @@ use std::fmt::Write;
 use ty_python_semantic::pull_types::pull_types;
 use ty_python_semantic::types::check_types;
 use ty_python_semantic::{
-    Program, ProgramSettings, PythonEnvironment, PythonPlatform, PythonVersionSource,
-    PythonVersionWithSource, SearchPathSettings, SysPrefixPathOrigin,
+    Module, Program, ProgramSettings, PythonEnvironment, PythonPlatform, PythonVersionSource,
+    PythonVersionWithSource, SearchPath, SearchPathSettings, SysPrefixPathOrigin, list_modules,
+    resolve_module,
 };
 
 mod assertion;
@@ -58,7 +59,7 @@ pub fn run(
     for test in suite.tests() {
         if filter
             .as_ref()
-            .is_some_and(|f| !test.uncontracted_name().contains(f))
+            .is_some_and(|f| !(test.uncontracted_name().contains(f) || test.name() == *f))
         {
             continue;
         }
@@ -68,13 +69,16 @@ pub fn run(
             Log::Filter(filter) => setup_logging_with_filter(filter),
         });
 
-        if let Err(failures) = run_test(&mut db, relative_fixture_path, snapshot_path, &test) {
-            any_failures = true;
+        let failures = run_test(&mut db, relative_fixture_path, snapshot_path, &test);
+        let inconsistencies = run_module_resolution_consistency_test(&db);
+        let this_test_failed = failures.is_err() || inconsistencies.is_err();
+        any_failures = any_failures || this_test_failed;
 
-            if output_format.is_cli() {
-                println!("\n{}\n", test.name().bold().underline());
-            }
+        if this_test_failed && output_format.is_cli() {
+            println!("\n{}\n", test.name().bold().underline());
+        }
 
+        if let Err(failures) = failures {
             let md_index = LineIndex::from_source_text(&source);
 
             for test_failures in failures {
@@ -100,20 +104,33 @@ pub fn run(
                     }
                 }
             }
-
-            let escaped_test_name = test.name().replace('\'', "\\'");
-
-            if output_format.is_cli() {
-                println!(
-                    "\nTo rerun this specific test, set the environment variable: {}='{escaped_test_name}'",
-                    EnvVars::MDTEST_TEST_FILTER,
-                );
-                println!(
-                    "{}='{escaped_test_name}' cargo test -p ty_python_semantic --test mdtest -- {test_name}",
-                    EnvVars::MDTEST_TEST_FILTER,
-                );
+        }
+        if let Err(inconsistencies) = inconsistencies {
+            any_failures = true;
+            for inconsistency in inconsistencies {
+                match output_format {
+                    OutputFormat::Cli => {
+                        let info = relative_fixture_path.to_string().cyan();
+                        println!("  {info} {inconsistency}");
+                    }
+                    OutputFormat::GitHub => {
+                        println!("::error file={absolute_fixture_path}::{inconsistency}");
+                    }
+                }
             }
         }
+
+        if this_test_failed && output_format.is_cli() {
+            let escaped_test_name = test.name().replace('\'', "\\'");
+            println!(
+                "\nTo rerun this specific test, set the environment variable: {}='{escaped_test_name}'",
+                EnvVars::MDTEST_TEST_FILTER,
+            );
+            println!(
+                "{}='{escaped_test_name}' cargo test -p ty_python_semantic --test mdtest -- {test_name}",
+                EnvVars::MDTEST_TEST_FILTER,
+            );
+        }
     }
 
     println!("\n{}\n", "-".repeat(50));
@@ -167,6 +184,8 @@ fn run_test(
     let project_root = SystemPathBuf::from("/src");
     db.create_directory_all(&project_root)
         .expect("Creating the project root to succeed");
+    db.files()
+        .try_add_root(db, &project_root, FileRootKind::Project);
 
     let src_path = project_root.clone();
     let custom_typeshed_path = test.configuration().typeshed();
@@ -222,7 +241,9 @@ fn run_test(
 
             db.write_file(&full_path, &embedded.code).unwrap();
 
-            if !(full_path.starts_with(&src_path) && matches!(embedded.lang, "py" | "pyi")) {
+            if !(full_path.starts_with(&src_path)
+                && matches!(embedded.lang, "py" | "python" | "pyi"))
+            {
                 // These files need to be written to the file system (above), but we don't run any checks on them.
                 return None;
             }
@@ -238,6 +259,8 @@ fn run_test(
 
     // Create a custom typeshed `VERSIONS` file if none was provided.
     if let Some(typeshed_path) = custom_typeshed_path {
+        db.files()
+            .try_add_root(db, typeshed_path, FileRootKind::LibrarySearchPath);
         if !has_custom_versions_file {
             let versions_file = typeshed_path.join("stdlib/VERSIONS");
             let contents = typeshed_files
@@ -283,6 +306,7 @@ fn run_test(
             extra_paths: configuration.extra_paths().unwrap_or_default().to_vec(),
             custom_typeshed: custom_typeshed_path.map(SystemPath::to_path_buf),
             site_packages_paths,
+            real_stdlib_path: None,
         }
         .to_search_paths(db.system(), db.vendored())
         .expect("Failed to resolve search path settings"),
@@ -405,6 +429,92 @@ fn run_test(
     }
 }
 
+/// Reports an inconsistency between "list modules" and "resolve module."
+///
+/// Values of this type are only constructed when `from_list` and
+/// `from_resolve` are not equivalent.
+struct ModuleInconsistency<'db> {
+    db: &'db db::Db,
+    /// The module returned from `list_module`.
+    from_list: Module<'db>,
+    /// The module returned, if any, from `resolve_module`.
+    from_resolve: Option<Module<'db>>,
+}
+
+/// Tests that "list modules" is consistent with "resolve module."
+///
+/// This only checks that everything returned by `list_module` is the
+/// identical module we get back from `resolve_module`. It does not
+/// check that all possible outputs of `resolve_module` are captured by
+/// `list_module`.
+fn run_module_resolution_consistency_test(db: &db::Db) -> Result<(), Vec<ModuleInconsistency<'_>>> {
+    let mut errs = vec![];
+    for from_list in list_modules(db) {
+        errs.push(match resolve_module(db, from_list.name(db)) {
+            None => ModuleInconsistency {
+                db,
+                from_list,
+                from_resolve: None,
+            },
+            Some(from_resolve) if from_list != from_resolve => ModuleInconsistency {
+                db,
+                from_list,
+                from_resolve: Some(from_resolve),
+            },
+            _ => continue,
+        });
+    }
+    if errs.is_empty() { Ok(()) } else { Err(errs) }
+}
+
+impl std::fmt::Display for ModuleInconsistency<'_> {
+    fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+        fn fmt_module(
+            db: &db::Db,
+            f: &mut std::fmt::Formatter,
+            module: &Module<'_>,
+        ) -> std::fmt::Result {
+            let name = module.name(db);
+            let path = module
+                .file(db)
+                .map(|file| file.path(db).to_string())
+                .unwrap_or_else(|| "N/A".to_string());
+            let search_path = module
+                .search_path(db)
+                .map(SearchPath::to_string)
+                .unwrap_or_else(|| "N/A".to_string());
+            let known = module
+                .known(db)
+                .map(|known| known.to_string())
+                .unwrap_or_else(|| "N/A".to_string());
+            write!(
+                f,
+                "Module(\
+                   name={name}, \
+                   file={path}, \
+                   kind={kind:?}, \
+                   search_path={search_path}, \
+                   known={known}\
+                 )",
+                kind = module.kind(db),
+            )
+        }
+        write!(f, "Found ")?;
+        fmt_module(self.db, f, &self.from_list)?;
+        match self.from_resolve {
+            None => write!(
+                f,
+                " when listing modules, but `resolve_module` returned `None`",
+            )?,
+            Some(ref got) => {
+                write!(f, " when listing modules, but `resolve_module` returned ",)?;
+                fmt_module(self.db, f, got)?;
+            }
+        }
+        Ok(())
+    }
+}
+
 type Failures = Vec<FileFailures>;
 
 /// The failures for a single file in a test by line number.
